#include <ccblkfn.h>
#include "adi_initialize.h"

/* CCLK frequency (HZ) */
#define BFIN_CORE_CLOCK			(500000000)
/* SCLK0 frequency (HZ) */
#define BFIN_SCLK0_CLOCK		(250000000)
/* SCLK1 frequency (HZ) */
#define BFIN_SCLK1_CLOCK		(250000000)

#define CONFIG_PLL_MULT			(20)
#define CONFIG_PLL_CLKIN_HALF	(0)
#define CONFIG_PLL_CCLK_DIV		(1)
#define CONFIG_PLL_SCLK_DIV		(2)
#define CONFIG_PLL_SCLK0_DIV	(1)
#define CONFIG_PLL_SCLK1_DIV	(1)
#define CONFIG_PLL_DCLK_DIV		(2)
#define CONFIG_PLL_OCLK_DIV		(16)

#define REG_CGU0_DIV_VAL \
	((CONFIG_PLL_CCLK_DIV  << BITP_CGU_DIV_CSEL)   | \
	 (CONFIG_PLL_SCLK0_DIV << BITP_CGU_DIV_S0SEL)  | \
	 (CONFIG_PLL_SCLK_DIV  << BITP_CGU_DIV_SYSSEL) | \
	 (CONFIG_PLL_SCLK1_DIV << BITP_CGU_DIV_S1SEL)  | \
	 (CONFIG_PLL_DCLK_DIV  << BITP_CGU_DIV_DSEL)   | \
	 (CONFIG_PLL_OCLK_DIV  << BITP_CGU_DIV_OSEL))

#define REG_CGU0_CTL_VAL \
	((CONFIG_PLL_MULT << BITP_CGU_CTL_MSEL) | (CONFIG_PLL_CLKIN_HALF << BITP_CGU_CTL_DF))

void bfin_cgu_pll_set(void)
{
	if((*pREG_CGU0_CTL & (BITM_CGU_CTL_MSEL | BITM_CGU_CTL_DF)) != REG_CGU0_CTL_VAL)
	{
		*pREG_CGU0_DIV = REG_CGU0_DIV_VAL;
		*pREG_CGU0_CTL = REG_CGU0_CTL_VAL;
		while ((*pREG_CGU0_STAT & (BITM_CGU_STAT_CLKSALGN | BITM_CGU_STAT_PLLBP)) ||
			  !(*pREG_CGU0_STAT & BITM_CGU_STAT_PLOCK))
		{
			continue;
		}
	}

	*pREG_CGU0_DIV = REG_CGU0_DIV_VAL | BITM_CGU_DIV_UPDT;
	while(*pREG_CGU0_STAT & BITM_CGU_STAT_CLKSALGN)
		continue;

	if((*pREG_DMC0_STAT & BITM_DMC_STAT_MEMINITDONE) != 0u)
	{
		*pREG_DMC0_CTL &= ~BITP_DMC_CTL_SRREQ;
		ssync();
		while((*pREG_DMC0_STAT & BITM_DMC_STAT_SRACK) != 0u)
			continue;
	}
}

/* {[[g(115200bps) */
#define UART0_BAUDRATE	(115200)
/* UART0_DLL/UART0_DLH̐ݒl */
#define UART0_DIVISOR	(BFIN_SCLK0_CLOCK/16/UART0_BAUDRATE)

static void s_uart0_init(void)
{
	*pREG_PORTD_FER = BITM_PORT_FER_SET_PX8 | BITM_PORT_FER_SET_PX7;
	*pREG_PORTD_MUX = (1 << BITP_PORT_MUX_MUX7) | (1 << BITP_PORT_MUX_MUX8);

	*pREG_UART0_CTL = 0;
	ssync();
	*pREG_UART0_CLK = UART0_DIVISOR;
	*pREG_UART0_CTL = ENUM_UART_CTL_PARITY_DIS | ENUM_UART_CTL_NO_EXTRA_STB | ENUM_UART_CTL_WL8BITS | ENUM_UART_CTL_UART_MODE | ENUM_UART_CTL_CLK_EN;
	ssync();
}

static void s_uart0_loopback(void)
{
	uint32_t l_rdata;

	while((*pREG_UART0_STAT & BITM_UART_STAT_DR) == ENUM_UART_STAT_NO_DATA) ssync();
	l_rdata = *pREG_UART0_RBR;
	while((*pREG_UART0_STAT & BITM_UART_STAT_THRE) == ENUM_UART_STAT_THR_NOT_EMPTY) ssync();
	*pREG_UART0_THR = l_rdata;
	ssync();
}


int main(void)
{
	/* Initialize managed drivers and/or services */
	adi_initComponents();
	bfin_cgu_pll_set();

	s_uart0_init();
	while(1)
	{
		s_uart0_loopback();
	}
}

